package tech.xiongweixp.pptassistant

import android.bluetooth.BluetoothManager
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import android.os.Bundle
import android.support.wearable.activity.WearableActivity
import android.net.wifi.WifiManager
import android.os.Handler
import android.os.Message
import android.support.wear.widget.BoxInsetLayout
import android.util.Log
import android.view.KeyEvent
import android.view.MotionEvent
import android.view.View
import android.view.WindowManager
import android.view.animation.AnimationUtils
import android.widget.*
import kotlinx.coroutines.*
import kotlin.concurrent.*
import java.io.InputStreamReader
import java.net.*
import android.view.animation.LinearInterpolator
import kotlinx.android.synthetic.main.activity_main.*
import java.io.IOException
import java.lang.IllegalArgumentException


class MainActivity : WearableActivity() {
    companion object {
        fun int2ip(ip:Int) = "${ip and 0xff}.${(ip shr 8) and 0xff}.${(ip shr 16) and 0xff}.${(ip shr 24) and 0xff}"
        fun int2sub(ip:Int) = "${ip and 0xff}.${(ip shr 8) and 0xff}.${(ip shr 16) and 0xff}"
        const val NETWORK_CONNECTIVITY_TIMEOUT_MS=30000L
        const val MESSAGE_CONNECTIVITY_TIMEOUT = 1
    }

    private var hostIp:String=""
    private var hostSub:String=""

    private val ipList=ArrayList<HostModel>()
    private var ipAdapter:IpListAdapter?=null

    private var text:TextView?=null
    private var welcome_pane:LinearLayout?=null
    private var scan_pane:LinearLayout?=null
    private var controlStart_pane:LinearLayout?=null
    private var control_pane:LinearLayout?=null
    private var main_pane:BoxInsetLayout?=null
    private var runButton: Button?=null
    private var presentButton: ImageView?=null
    private var time_lable: TextView?=null
    private var listView: ListView?=null
    private var imageView2: ImageView?=null

    private var controlStart = false
    private var stopScan = false

    private val controlUtil = ControlUtil.getInstance()

    private var h=0
    private var m=0
    private var s=0
    private var motionRecordY = 0.0f

    private lateinit var scanSocketServer:ServerSocket

    //请求使用wifi的管理器
    private lateinit var connectivityManager: ConnectivityManager
    private lateinit var wifiManager:WifiManager
    private val timeoutHandler=MyHandler()

    //广播接收线程
    private val scanResultThread = GlobalScope.launch(start = CoroutineStart.LAZY) {
        scanSocketServer=ServerSocket(1234)
        while(true){
            yield()
            try{
                val socket = scanSocketServer.accept()
                GlobalScope.launch {
                    socket.use{
                        val  reader = InputStreamReader(socket.getInputStream(),"utf8")
                        reader.use {
                            val  receivText = reader.readText()
                            val info = receivText.split(",")
                            val hostName = info[0]
                            val hostIp = info[1]
                            val ips=hostIp.split("|")
                            for(ip in  ips){
                                if(ip.startsWith(hostSub)){
                                    updateItem(hostName, ip)
                                }
                            }
                        }
                    }
                }
            }catch(e  : SocketException){

            }
        }
    }

    private val scanThread = GlobalScope.launch(start = CoroutineStart.LAZY){
        //启动接收线程
        scanResultThread.start()
        //使用UDP发送广播进行发现,使用随机端口
        var port=(40000..41000).random()
        var udpSocket:DatagramSocket?=null
        while(udpSocket==null) {
            try {
                udpSocket = DatagramSocket(port)
            } catch (e: BindException) {
                Log.e("BIND UDP","Port $port is used", e)
                port++
            }
        }
        val broadcastMsg="requestip $hostIp".toByteArray()
        //2s one time
        var times=0
        while(true) {
            times++
            //持续广播1分钟
            if(stopScan||times>30){
                break
            }
            udpSocket.broadcast = true
            udpSocket.reuseAddress = true
            val packet = DatagramPacket(broadcastMsg, broadcastMsg.size)
            packet.address = InetAddress.getByName("$hostSub.255")
            packet.port = 5678
            try {
                udpSocket.send(packet)
            }catch(e: IOException){
                Log.e("UDP BROADCAST","broadcast error",e)
            }
            delay(2000)
        }
        udpSocket.close()
        //如果没有发现计算机
        if(ipList.size==0){
            runOnUiThread{
                imageView2?.visibility=View.INVISIBLE
                text?.setText(R.string.no_computer)
            }
        }
    }

    private fun  updateItem(name:String,  ip:String){
        runOnUiThread {
            var contain=false
            for(item in  ipList){
                if(item.ip==ip){
                    contain=true
                    break
                }
            }
            if(!contain) {
                ipList.add(HostModel(name,ip))
                ipAdapter?.notifyDataSetChanged()
            }
        }
    }

    private fun initViewContent(){
        listView = findViewById<ListView>(R.id.listView)
        ipAdapter=IpListAdapter(this, ipList)
        listView?.adapter=ipAdapter
        welcome_pane=findViewById<LinearLayout>(R.id.welcome_pane)
        scan_pane=findViewById<LinearLayout>(R.id.scan_pane)
        controlStart_pane=findViewById<LinearLayout>(R.id.controlStart_pane)
        control_pane = findViewById<LinearLayout>(R.id.control_pane)
        main_pane = findViewById<BoxInsetLayout>(R.id.main_pane)
        runButton=findViewById<Button>(R.id.runButton)
        presentButton=findViewById<ImageView>(R.id.presentButton)
        time_lable = findViewById<TextView>(R.id.time_label)
        text = findViewById<TextView>(R.id.text)
    }

    private  val avilableWifiCallback  = object:ConnectivityManager.NetworkCallback(){
        override fun onAvailable(network: Network?) {
            timeoutHandler.removeMessages(MESSAGE_CONNECTIVITY_TIMEOUT)
            if(connectivityManager.bindProcessToNetwork(network)) {
                runOnUiThread{
                    val connectInfo = wifiManager.connectionInfo
                    val ip = connectInfo.ipAddress
                    hostIp = int2ip(ip)
                    hostSub = int2sub(ip)

                    if (hostIp == "0.0.0.0") {
                        imageView2?.visibility=View.INVISIBLE
                        text?.setText(R.string.no_wifi)
                    } else {
                        text?.setText(R.string.scanning)
                        scanThread.start()
                    }
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        //保持屏幕常亮
        window.setFlags(
            WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON,
            WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        setTheme(R.style.AppTheme)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        initViewContent()

        wifiManager = applicationContext.getSystemService(WIFI_SERVICE) as WifiManager
        if(!wifiManager.isWifiEnabled) {
            wifiManager.isWifiEnabled = true
        }
        connectivityManager = getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager


        imageView2 = findViewById(R.id.loadingImage)
        val animat = AnimationUtils.loadAnimation(this,R.anim.loading)
        val lin = LinearInterpolator()
        animat.interpolator = lin
        imageView2?.animation=animat
        //扫描局域网
        runButton?.setOnClickListener {
            welcome_pane?.visibility = View.INVISIBLE
            scan_pane?.visibility = View.VISIBLE
            text?.setText(R.string.connecting)

            val request: NetworkRequest = NetworkRequest.Builder().run {
                addTransportType(NetworkCapabilities.TRANSPORT_WIFI)
                addCapability(NetworkCapabilities.NET_CAPABILITY_NOT_METERED)
                addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
                build()
            }
            connectivityManager.requestNetwork(request, avilableWifiCallback)
            timeoutHandler.sendMessageDelayed(
                timeoutHandler.obtainMessage(MESSAGE_CONNECTIVITY_TIMEOUT),
                NETWORK_CONNECTIVITY_TIMEOUT_MS
            )
        }

        //连接计算机
        listView?.setOnItemClickListener { _, _, position, _ ->
            stopScan = true
            scanResultThread.cancel()
            scanSocketServer.close()
            val host = ipList[position]
            controlUtil?.connect(host.ip)
            scan_pane?.visibility = View.INVISIBLE
            controlStart_pane?.visibility = View.VISIBLE
        }

        //开始演示
        presentButton?.setOnClickListener{
            controlUtil?.start()
            h=0
            m=0
            s=0
            controlStart_pane?.visibility = View.INVISIBLE
            control_pane?.visibility = View.VISIBLE
            controlStart=true
            timer(period = 1000){
                s++
                if(s==60){
                    m++
                    s=0
                }
                if(m==60){
                    m=0
                    h++
                }
                runOnUiThread{
                    time_lable?.text="${String.format("%02d",h)}:${String.format("%02d",m)}:${String.format("%02d",s)}"
                }
            }
        }

        time_lable?.setOnLongClickListener {
            controlUtil?.stop()
            control_pane?.visibility=View.INVISIBLE
            controlStart_pane?.visibility=View.VISIBLE
            false
        }

        // Enables Always-on
        setAmbientEnabled()
    }

    override fun onDestroy() {
        if(::scanSocketServer.isInitialized) {
            scanSocketServer.close()
        }
        stopScan=true
        //注销wifi连接
        connectivityManager.bindProcessToNetwork(null)
        try {
            connectivityManager.unregisterNetworkCallback(avilableWifiCallback)
        }catch(e:IllegalArgumentException){

        }
        super.onDestroy()
    }

    //滑动控制
    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        when(ev?.action) {
            MotionEvent.ACTION_DOWN -> {
                motionRecordY = ev.y
            }
            MotionEvent.ACTION_UP -> {
                val moveY = ev.y - motionRecordY
                if (controlStart && moveY < -100) {
                    controlUtil?.previous()
                }else if(controlStart && moveY>100){
                    controlUtil?.next()
                }
            }
        }
        return super.dispatchTouchEvent(ev)
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        when (keyCode) {
            KeyEvent.KEYCODE_NAVIGATE_NEXT -> {
                if(controlStart){
                    controlUtil?.next()
                }
            }
            KeyEvent.KEYCODE_NAVIGATE_PREVIOUS  -> {
                if(controlStart){
                    controlUtil?.previous()
                }
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    private inner class MyHandler : Handler() {
        override fun handleMessage(msg: Message) {

            when (msg.what) {
                MESSAGE_CONNECTIVITY_TIMEOUT -> {
                    // unregister the network
                    runOnUiThread {
                        imageView2?.visibility=View.INVISIBLE
                        textView2?.setText(R.string.no_wifi)
                    }
                }
            }
        }
    }
}
